# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Communities Service API config."""

from invenio_i18n import lazy_gettext as _
from invenio_records_resources.services import FileServiceConfig
from invenio_records_resources.services.base.config import (
    ConfiguratorMixin,
    FromConfig,
    FromConfigSearchOptions,
    SearchOptionsMixin,
)
from invenio_records_resources.services.files.links import FileLink
from invenio_records_resources.services.records.config import RecordServiceConfig
from invenio_records_resources.services.records.config import (
    SearchOptions as SearchOptionsBase,
)
from invenio_records_resources.services.records.links import pagination_links
from invenio_records_resources.services.records.params import (
    FacetsParam,
    PaginationParam,
    QueryStrParam,
)

from invenio_communities.communities.records.api import Community
from invenio_communities.communities.services import facets
from invenio_communities.communities.services.results import (
    CommunityFeaturedList,
    CommunityItem,
    CommunityListResult,
    FeaturedCommunityItem,
)

from ...permissions import CommunityPermissionPolicy, can_perform_action
from ..schema import CommunityFeaturedSchema, CommunitySchema, TombstoneSchema
from .components import DefaultCommunityComponents
from .links import CommunityLink
from .search_params import IncludeDeletedCommunitiesParam, StatusParam
from .sort import CommunitiesSortParam


class SearchOptions(SearchOptionsBase, SearchOptionsMixin):
    """Search options."""

    sort_featured = {
        "title": _("Featured"),
        "fields": [
            {
                "featured.past": {
                    "order": "desc",
                }
            }
        ],
    }

    facets = {"type": facets.type, "visibility": facets.visibility}
    params_interpreters_cls = [
        QueryStrParam,
        PaginationParam,
        CommunitiesSortParam,
        FacetsParam,
        StatusParam,
        IncludeDeletedCommunitiesParam,
    ]


class CommunityServiceConfig(RecordServiceConfig, ConfiguratorMixin):
    """Communities service configuration."""

    service_id = "communities"

    # Common configuration
    permission_policy_cls = FromConfig(
        "COMMUNITIES_PERMISSION_POLICY", default=CommunityPermissionPolicy
    )
    # Record specific configuration
    record_cls = Community
    result_item_cls = CommunityItem
    result_list_cls = CommunityListResult
    indexer_queue_name = "communities"

    # Search configuration
    search = FromConfigSearchOptions(
        "COMMUNITIES_SEARCH",
        "COMMUNITIES_SORT_OPTIONS",
        "COMMUNITIES_FACETS",
        search_option_cls=SearchOptions,
    )

    # Service schema
    schema = CommunitySchema
    schema_featured = CommunityFeaturedSchema
    schema_tombstone = TombstoneSchema

    result_list_cls_featured = CommunityFeaturedList
    result_item_cls_featured = FeaturedCommunityItem

    links_item = {
        "self": CommunityLink("{+api}/communities/{id}"),
        "self_html": CommunityLink("{+ui}/communities/{slug}"),
        "settings_html": CommunityLink("{+ui}/communities/{slug}/settings"),
        "logo": CommunityLink("{+api}/communities/{id}/logo"),
        "rename": CommunityLink("{+api}/communities/{id}/rename"),
        "members": CommunityLink("{+api}/communities/{id}/members"),
        "public_members": CommunityLink("{+api}/communities/{id}/members/public"),
        "invitations": CommunityLink("{+api}/communities/{id}/invitations"),
        "requests": CommunityLink("{+api}/communities/{id}/requests"),
        "records": CommunityLink("{+api}/communities/{id}/records"),
    }

    action_link = CommunityLink(
        "{+api}/communities/{id}/{action_name}", when=can_perform_action
    )

    links_search = pagination_links("{+api}/communities{?args*}")
    links_featured_search = pagination_links("{+api}/communities/featured{?args*}")
    links_user_search = pagination_links("{+api}/user/communities{?args*}")
    links_community_requests_search = pagination_links(
        "{+api}/communities/{community_id}/requests{?args*}"
    )

    available_actions = [
        {"action_name": "featured", "action_permission": "featured_create"}
    ]

    # Service components
    components = FromConfig(
        "COMMUNITIES_SERVICE_COMPONENTS", default=DefaultCommunityComponents
    )


class CommunityFileServiceConfig(FileServiceConfig, ConfiguratorMixin):
    """Configuration for community files."""

    record_cls = Community
    permission_policy_cls = FromConfig(
        "COMMUNITIES_PERMISSION_POLICY", default=CommunityPermissionPolicy
    )
    file_links_item = {
        "self": FileLink("{+api}/communities/{id}/logo"),
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 TU Wien.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Tests for the tombstone field and community deletion field."""

import datetime

import pytest
from invenio_requests.resolvers.registry import ResolverRegistry

from invenio_communities.communities.records.api import Community
from invenio_communities.communities.records.systemfields.deletion_status import (
    CommunityDeletionStatus,
    CommunityDeletionStatusEnum,
)
from invenio_communities.communities.records.systemfields.tombstone import Tombstone

#
# Tombstone
#


def test_tombstone_creation(app):
    """Test the normal creation of a tombstone."""
    t = Tombstone({})
    assert t.removed_by is None
    assert t.removal_reason is None
    assert t.note == ""
    assert t.removal_date
    assert isinstance(t.removal_date, str)
    assert t.citation_text == ""
    assert t.is_visible

    data = {
        "removed_by": {"user": "1"},
        "removal_reason": {"id": "spam"},
        "note": "nothing in particular",
        "removal_date": datetime.datetime.utcnow(),
        "citation_text": "No citation available, sorry",
        "is_visible": False,
    }

    t = Tombstone(data)
    assert t.removed_by == data["removed_by"]
    assert t.removal_reason == data["removal_reason"]
    assert t.note == data["note"]
    assert t.removal_date == data["removal_date"].isoformat()
    assert t.citation_text == data["citation_text"]
    assert not t.is_visible


def test_tombstone_invalid_removed_by(app):
    """Test the failure of tombstone creation if the `removed_by` entry is invalid."""
    for invalid_value in [[], datetime.datetime.utcnow()]:
        with pytest.raises(ValueError):
            Tombstone({"removed_by": invalid_value})


def test_tombstone_valid_removed_by(app, users):
    """Test various ways to set the `removed_by` value for a tombstone."""
    user = users["owner"].user

    # if the assigned value is an int or string, we assume it's a user ID
    t = Tombstone({})
    t.removed_by = user.id
    assert t.removed_by == {"user": str(user.id)}
    assert t.removed_by_proxy.resolve() == user

    # this comes in handy for setting the system to be the one who deleted the community
    t.removed_by = "system"
    assert t.removed_by == {"user": "system"}
    assert t.removed_by_proxy.resolve() == {
        "id": "system",
        "is_ghost": True,
        "profile": {"full_name": "System"},
        "username": "System",
    }

    # None should work as expected
    t.removed_by = None
    assert t.removed_by is None
    assert t.removed_by_proxy is None

    # setting a referenceable entity should use the `ResolverRegistry`
    t.removed_by = user
    assert ResolverRegistry.reference_entity(user) is not None
    assert t.removed_by == {"user": str(user.id)}
    assert t.removed_by_proxy.resolve() == user


#
# Community deletion status
#


def test_community_deletion_status_default(app):
    """Test the default value of the deletion status."""
    deletion_status = CommunityDeletionStatus(None)
    assert deletion_status.status == CommunityDeletionStatusEnum.PUBLISHED.value
    assert not deletion_status.is_deleted


def test_community_deletion_status_valid_values(app):
    """Test setting the deletion status to valid values."""
    deletion_status = CommunityDeletionStatus(CommunityDeletionStatusEnum.PUBLISHED)
    assert deletion_status.status == CommunityDeletionStatusEnum.PUBLISHED.value
    assert not deletion_status.is_deleted

    deletion_status.status = CommunityDeletionStatusEnum.DELETED
    assert deletion_status.status == CommunityDeletionStatusEnum.DELETED.value
    assert deletion_status.is_deleted

    deletion_status.status = CommunityDeletionStatusEnum.MARKED
    assert deletion_status.status == CommunityDeletionStatusEnum.MARKED.value
    assert deletion_status.is_deleted


def test_community_deletion_status_invalid_values(app):
    """Test setting the community deletion status to invalid values."""
    for invalid_value in ["s", 1, []]:
        with pytest.raises(ValueError):
            CommunityDeletionStatus(invalid_value)


def test_community_deletion_status(app, location, minimal_community):
    """Test if changing the community deletion status impacts the DB model."""
    minimal_community = minimal_community.copy()
    slug = minimal_community.pop("slug")

    comm = Community.create(minimal_community, slug=slug)
    assert comm.deletion_status.status == CommunityDeletionStatusEnum.PUBLISHED.value
    assert comm.deletion_status._status == CommunityDeletionStatusEnum.PUBLISHED
    assert (
        CommunityDeletionStatus(comm.model.deletion_status)
        == CommunityDeletionStatusEnum.PUBLISHED
    )
    assert not comm.deletion_status.is_deleted
    assert not comm.is_deleted

    # NOTE that the `community.deletion_status.is_deleted` relates to the
    #      community deletion workflow, while `community.is_deleted` is defined
    #      in `invenio_records.api.RecordBase` and marks the absolute deletion
    #      of the community
    comm.deletion_status = CommunityDeletionStatusEnum.DELETED
    assert comm.deletion_status.status == CommunityDeletionStatusEnum.DELETED.value
    assert comm.deletion_status._status == CommunityDeletionStatusEnum.DELETED
    assert (
        CommunityDeletionStatusEnum(comm.model.deletion_status)
        == CommunityDeletionStatusEnum.DELETED
    )
    assert comm.deletion_status.is_deleted
    assert not comm.is_deleted



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test community members resources."""

import pytest
from invenio_access.permissions import system_identity
from invenio_requests.records.api import RequestEvent


#
# Fixtures
#
@pytest.fixture()
def community_id(community):
    return community._record.id


@pytest.fixture(scope="function")
def group_data(group):
    return {
        "members": [{"type": "group", "id": group.name}],
        "role": "reader",
    }


@pytest.fixture(scope="function")
def new_user_data(new_user):
    return {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
        "message": "Welcome to the club!",
    }


#
# Add
#
def test_add(client, headers, community_id, owner, group_data, db):
    """Test add REST API."""
    client = owner.login(client)
    r = client.post(
        f"/communities/{community_id}/members", headers=headers, json=group_data
    )
    assert r.status_code == 204


def test_add_denied(client, headers, community_id, group_data, new_user):
    """Test add REST API."""
    client = new_user.login(client)
    r = client.post(
        f"/communities/{community_id}/members", headers=headers, json=group_data
    )
    assert r.status_code == 403


def test_add_bad_data(client, headers, community_id, owner):
    """Test add REST API."""
    client = owner.login(client)
    r = client.post(
        f"/communities/{community_id}/members", headers=headers, json={"members": []}
    )
    assert r.status_code == 400


def test_add_invalid_member(client, headers, community_id, owner, group_data):
    """Test add REST API."""
    client = owner.login(client)
    group_data["members"][0]["id"] = "invalid"
    r = client.post(
        f"/communities/{community_id}/members", headers=headers, json=group_data
    )
    assert r.status_code == 400


def test_add_duplicate(client, headers, community_id, owner, group_data, db):
    """Test add REST API."""
    client = owner.login(client)
    r = client.post(
        f"/communities/{community_id}/members", headers=headers, json=group_data
    )
    assert r.status_code == 204
    r = client.post(
        f"/communities/{community_id}/members", headers=headers, json=group_data
    )
    assert r.status_code == 400


#
# Invite
#
def test_invite(client, headers, community_id, owner, new_user_data, db):
    """Test invite REST API."""
    client = owner.login(client)
    r = client.post(
        f"/communities/{community_id}/invitations",
        headers=headers,
        json=new_user_data,
    )
    assert r.status_code == 204

    RequestEvent.index.refresh()
    r = client.get(f"/communities/{community_id}/invitations", headers=headers)
    assert r.status_code == 200
    request_id = r.json["hits"]["hits"][0]["request"]["id"]

    # check the request
    r = client.get(f"/requests/{request_id}", headers=headers)
    assert r.status_code == 200
    request = r.json
    assert 'You will join as "Reader"' in request["description"]

    # check the timeline
    r = client.get(f"/requests/{request_id}/timeline", headers=headers)
    assert r.status_code == 200
    assert r.json["hits"]["total"] == 1  # invite message
    assert r.json["hits"]["hits"][0]["payload"]["content"] == new_user_data["message"]


def test_invite_deny(client, headers, community_id, new_user, new_user_data, db):
    """Test invite REST API."""
    client = new_user.login(client)
    r = client.post(
        f"/communities/{community_id}/invitations", headers=headers, json=new_user_data
    )
    assert r.status_code == 403


#
# Update
#
def test_update(client, headers, community_id, owner, public_reader):
    """Test update of members."""
    client = owner.login(client)
    data = {
        "members": [{"type": "user", "id": str(public_reader.id)}],
        "visible": False,
        "role": "curator",
    }
    r = client.put(
        f"/communities/{community_id}/members",
        headers=headers,
        json=data,
    )
    assert r.status_code == 204


def test_update_invite(client, headers, community_id, owner, new_user_data, db):
    """Test update of members."""
    client = owner.login(client)
    new_user_data.pop("message")  # not accepted by update
    r = client.post(
        f"/communities/{community_id}/invitations",
        headers=headers,
        json=new_user_data,
    )
    assert r.status_code == 204

    # Update the invite
    new_user_data["role"] = "curator"
    r = client.put(
        f"/communities/{community_id}/invitations",
        headers=headers,
        json=new_user_data,
    )
    assert r.status_code == 204


#
# Delete
#
def test_delete(client, headers, community_id, owner, public_reader):
    """Test delete of members."""
    client = public_reader.login(client)
    data = {
        "members": [{"type": "user", "id": str(public_reader.id)}],
    }
    r = client.delete(
        f"/communities/{community_id}/members",
        headers=headers,
        json=data,
    )
    assert r.status_code == 204


@pytest.fixture(scope="function")
def extra_user(app, db, UserFixture, member_service, community):
    """Add a reader member with public visibility."""
    u = UserFixture(
        email=f"extra@newuser.org",
        password="newuser",
        user_profile={
            "full_name": "Should be last",
        },
        preferences={
            "visibility": "public",
            "email_visibility": "restricted",
        },
        active=True,
        confirmed=True,
    )
    u.create(app, db)
    data = {
        "members": [{"type": "user", "id": str(u.id)}],
        "role": "reader",
        "visible": True,
    }

    member_service.add(system_identity, community._record.id, data)
    return u


#
# Search and serialization
#
def test_search(
    db,
    clean_index,
    client,
    headers,
    extra_user,
    community_id,
    owner,
    public_reader,
):
    """Search."""
    client = owner.login(client)
    r = client.get(
        f"/communities/{community_id}/members",
        headers=headers,
    )
    assert r.status_code == 200
    data = r.json
    assert data["sortBy"] == "name"
    assert data["hits"]["total"] == 3
    assert "role" in data["aggregations"]
    assert "visibility" in data["aggregations"]

    hit = data["hits"]["hits"][0]
    assert "role" in hit
    assert "visible" in hit
    assert "created" in hit
    assert "updated" in hit
    assert "revision_id" in hit
    assert "is_current_user" in hit
    assert "permissions" in hit
    assert hit["permissions"]["can_leave"] is False
    assert hit["permissions"]["can_delete"] is True
    assert hit["permissions"]["can_update_role"] is True
    assert hit["permissions"]["can_update_visible"] is True
    assert hit["visible"] is True

    hit = data["hits"]["hits"][1]  # should be last, test sorting
    assert hit["member"]["name"] == "Should be last"

    # third because of no name
    hit = data["hits"]["hits"][2]
    assert hit["is_current_user"] is True
    assert hit["permissions"]["can_leave"] is True
    assert hit["permissions"]["can_delete"] is False
    assert hit["permissions"]["can_update_role"] is False
    assert hit["permissions"]["can_update_visible"] is True
    assert hit["visible"] is False

    # Pagination params should be passed correctly as well.
    # see https://github.com/inveniosoftware/invenio-communities/issues/495
    r1 = client.get(
        f"/communities/{community_id}/members",
        headers=headers,
        query_string={"page": 1, "size": 1},
    ).json
    assert r1["hits"]["total"] == 3
    assert len(r1["hits"]["hits"]) == 1

    r2 = client.get(
        f"/communities/{community_id}/members",
        headers=headers,
        query_string={"page": 3, "size": 1},
    ).json
    assert r2["hits"]["total"] == 3
    assert len(r2["hits"]["hits"]) == 1
    assert r1["hits"]["hits"][0]["id"] != r2["hits"]["hits"][0]


def test_search_public(
    client, headers, community_id, new_user, public_reader, clean_index
):
    """Search public members."""
    client = new_user.login(client)
    r = client.get(
        f"/communities/{community_id}/members/public",
        headers=headers,
    )
    assert r.status_code == 200
    data = r.json
    assert data["sortBy"] == "name"
    assert data["hits"]["total"] == 1
    hit = data["hits"]["hits"][0]
    # Public view has no facets (because that would leak information on
    # roles/visible)
    assert "aggregations" not in data
    # A member in the public view should not leak below attributes:
    assert "role" not in hit
    assert "visible" not in hit
    assert "created" not in hit
    assert "updated" not in hit
    assert "revision_id" not in hit
    assert "is_current_user" not in hit
    assert "permissions" not in hit
    # A member do have:
    assert "member" in hit
    assert "id" in hit["member"]
    assert "type" in hit["member"]
    assert "name" in hit["member"]
    assert "description" in hit["member"]


def test_search_invitation(
    client, headers, community_id, owner, invite_user, db, clean_index
):
    """Search invitations."""
    client = owner.login(client)
    r = client.get(
        f"/communities/{community_id}/invitations",
        headers=headers,
    )
    assert r.status_code == 200
    data = r.json
    assert data["hits"]["total"] == 1
    assert "role" in data["aggregations"]
    assert "status" in data["aggregations"]
    hit = data["hits"]["hits"][0]
    assert "role" in hit
    assert "visible" in hit
    assert "created" in hit
    assert "updated" in hit
    assert "revision_id" in hit
    assert "request" in hit
    assert "id" in hit["request"]
    assert "status" in hit["request"]
    assert "expires_at" in hit["request"]
    assert hit["request"]["expires_at"] is not None
    assert "permissions" in hit
    assert hit["permissions"]["can_update_role"] is True
    assert hit["permissions"]["can_cancel"] is True


# TODO: member serialization/links
# TODO: request serialization/links
# TODO: community member can see info
# TODO: community non-member can't see info
# TODO: facet by role, facet by visibility, define sorts.
# TODO: same user can be invited to two different communities
# TODO: same user/group can be added to two different communities



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test community views."""

from invenio_communities.views.communities import _filter_roles


def _test_filter_roles(app, members, action, member_types, community_id):
    """Test _filter_roles func."""
    for role in app.config["COMMUNITIES_ROLES"]:
        role_name = role["name"]
        member = members[role_name]

        roles = _filter_roles(
            action, member_types, community_id, identity=member.identity
        )
        filtered = {role["name"] for role in roles}
        assert filtered == set(role.get("can_manage_roles", []))


def test_filter_roles_when_updating_roles(
    app, db, community_service, community, members
):
    """Test selectable roles when updating existing members' roles."""
    _test_filter_roles(
        app,
        members,
        "members_update",
        {"user", "group"},
        community.id,
    )


def test_filter_roles_when_inviting_members(
    app, db, community_service, community, members
):
    """Test selectable roles when inviting new members."""
    _test_filter_roles(
        app,
        members,
        "members_invite",
        {"user"},
        community.id,
    )


def test_filter_roles_when_adding_groups(
    app, db, community_service, community, members
):
    """Test selectable roles when adding new groups."""
    _test_filter_roles(
        app,
        members,
        "members_add",
        {"group"},
        community.id,
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
# Copyright (C) 2023 TU Wien.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""UI community schema."""

from functools import partial

from flask import g
from flask_resources import BaseObjectSchema
from invenio_i18n import get_locale
from invenio_i18n import lazy_gettext as _
from invenio_records_resources.services.custom_fields import CustomFieldsSchemaUI
from invenio_vocabularies.contrib.awards.serializer import AwardL10NItemSchema
from invenio_vocabularies.contrib.funders.serializer import FunderL10NItemSchema
from invenio_vocabularies.resources import VocabularyL10Schema
from marshmallow import Schema, fields, post_dump
from marshmallow_utils.fields import FormatEDTF as FormatEDTF_

from invenio_communities.proxies import current_communities


def _community_permission_check(action, community, identity):
    """Check community permission for identity."""
    return current_communities.service.config.permission_policy_cls(
        action,
        community_id=getattr(community, "id", community["id"]),
        record=community,
    ).allows(identity)


def mask_removed_by(obj):
    """Mask information about who removed the community."""
    return_value = _("Unknown")
    removed_by = obj.get("removed_by", None)

    if removed_by is not None:
        user = removed_by.get("user", None)

        if user == "system":
            return_value = _("System (automatic)")
        elif user is not None:
            return_value = _("User")

    return return_value


# Partial to make short definitions in below schema.
FormatEDTF = partial(FormatEDTF_, locale=get_locale)


class TombstoneSchema(Schema):
    """Schema for a record tombstone."""

    removal_reason = fields.Nested(VocabularyL10Schema, attribute="removal_reason")

    note = fields.String(attribute="note")

    removed_by = fields.Function(mask_removed_by)

    removal_date_l10n_medium = FormatEDTF(attribute="removal_date", format="medium")

    removal_date_l10n_long = FormatEDTF(attribute="removal_date", format="long")

    citation_text = fields.String(attribute="citation_text")

    is_visible = fields.Boolean(attribute="is_visible")


class FundingSchema(Schema):
    """Schema for dumping types in the UI."""

    award = fields.Nested(AwardL10NItemSchema)
    funder = fields.Nested(FunderL10NItemSchema)


class UICommunitySchema(BaseObjectSchema):
    """Schema for dumping extra information of the community for the UI."""

    type = fields.Nested(VocabularyL10Schema, attribute="metadata.type")

    funding = fields.List(
        fields.Nested(FundingSchema()),
        attribute="metadata.funding",
    )

    tombstone = fields.Nested(TombstoneSchema, attribute="tombstone")

    # Custom fields
    custom_fields = fields.Nested(
        partial(CustomFieldsSchemaUI, fields_var="COMMUNITIES_CUSTOM_FIELDS")
    )

    permissions = fields.Method("get_permissions", dump_only=True)

    def get_permissions(self, obj):
        """Get permission."""
        can_include_directly = _community_permission_check(
            "include_directly", community=obj, identity=g.identity
        )
        can_update = _community_permission_check(
            "update", community=obj, identity=g.identity
        )
        return {"can_include_directly": can_include_directly, "can_update": can_update}

    @post_dump
    def hide_tombstone(self, obj, **kwargs):
        """Hide the tombstone information if it's not visible."""
        if not obj.get("tombstone", {}).get("is_visible", False):
            obj.pop("tombstone", None)

        return obj


class TypesSchema(Schema):
    """Schema for dumping types in the UI."""

    types = fields.List(fields.Nested(VocabularyL10Schema), attribute="types")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Fake demo records."""

import json
import random


def create_fake_community(faker):
    """Create fake communities for demo purposes."""
    data_to_use = {
        "access": {
            "visibility": random.choice(["public", "restricted"]),
            "member_policy": random.choice(["open", "closed"]),
            "record_policy": random.choice(["open", "closed"]),
        },
        "slug": faker.unique.domain_word(),
        "metadata": {
            "title": faker.sentence(nb_words=5, variable_nb_words=True),
            "description": faker.text(max_nb_chars=250),
            "type": {
                "id": random.choice(["organization", "event", "topic", "project"])
            },
            "curation_policy": faker.text(max_nb_chars=2000),
            "page": faker.text(max_nb_chars=2000),
            "website": "https://" + faker.domain_name(),  # fake.url()
            "organizations": [{"name": "CERN"}],
        },
    }

    return json.loads(json.dumps(data_to_use))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS bundles for communities.

You include one of the bundles in a page like the example below (using
``comunities`` bundle as an example):

.. code-block:: html

    {{ webpack['communities.js']}}

"""

from invenio_assets.webpack import WebpackThemeBundle

communities = WebpackThemeBundle(
    __name__,
    "assets",
    default="semantic-ui",
    themes={
        "semantic-ui": dict(
            entry={
                "invenio-communities-new": "./js/invenio_communities/community/new.js",
                "invenio-communities-privileges": "./js/invenio_communities/settings/privileges/index.js",
                "invenio-communities-profile": "./js/invenio_communities/settings/profile/index.js",
                "invenio-communities-curation-policy": "./js/invenio_communities/settings/curationPolicy/index.js",
                "invenio-communities-settings-pages": "./js/invenio_communities/settings/pages/index.js",
                "invenio-communities-requests": "./js/invenio_communities/requests/index.js",
                "invenio-communities-frontpage": "./js/invenio_communities/community/frontpage.js",
                "invenio-communities-search": "./js/invenio_communities/community/search.js",
                "invenio-communities-members": "./js/invenio_communities/members/members/member_view/index.js",
                "invenio-communities-members-manager": "./js/invenio_communities/members/members/manager_view/index.js",
                "invenio-communities-members-public": "./js/invenio_communities/members/members/public_view/index.js",
                "invenio-communities-invitations": "./js/invenio_communities/members/invitations/index.js",
                "invenio-communities-carousel": "./js/invenio_communities/community/communitiesCarousel/index.js",
                "invenio-communities-admin-search": "./js/invenio_communities/administration/search.js",
                "invenio-communities-featured": "./js/invenio_communities/community/featuredCommunities/index.js",
                "invenio-communities-admin-featured": "./js/invenio_communities/administration/details.js",
            },
            dependencies={
                "@ckeditor/ckeditor5-build-classic": "^16.0.0",
                "@ckeditor/ckeditor5-react": "^2.1.0",
                "@semantic-ui-react/css-patch": "^1.0.0",
                "react-router-dom": "^6.3.0",
                "react-invenio-forms": "^2.0.0",
                "axios": "^0.21.0",
                "formik": "^2.1.0",
                "i18next": "^20.3.0",
                "i18next-browser-languagedetector": "^6.1.0",
                "lodash": "^4.17.0",
                "luxon": "^1.23.0",
                "path": "^0.12.0",
                "prop-types": "^15.7.0",
                "qs": "^6.9.0",
                "react": "^16.13.0",
                "react-dom": "^16.13.0",
                "react-i18next": "^11.11.0",
                "react-redux": "^7.2.0",
                "react-searchkit": "^2.0.0",
                "redux": "^4.0.0",
                "redux-thunk": "^2.3.0",
                "semantic-ui-css": "^2.4.0",
                "semantic-ui-react": "^2.1.0",
                "yup": "^0.32.11",
            },
            aliases={
                # Define Semantic-UI theme configuration needed by
                # Invenio-Theme in order to build Semantic UI (in theme.js
                # entry point). theme.config itself is provided by
                # cookiecutter-invenio-rdm.
                "@js/invenio_communities": "js/invenio_communities",
                "@translations/invenio_communities": "translations/invenio_communities",
            },
        ),
    },
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Notification related utils for notifications."""

from invenio_notifications.models import Notification
from invenio_notifications.registry import EntityResolverRegistry
from invenio_notifications.services.builders import NotificationBuilder
from invenio_notifications.services.generators import EntityResolve, UserEmailBackend
from invenio_users_resources.notifications.filters import UserPreferencesRecipientFilter
from invenio_users_resources.notifications.generators import UserRecipient


class CommunityInvitationSubmittedNotificationBuilder(NotificationBuilder):
    """Notification builder for community invitation submit event."""

    type = "community-invitation.submit"

    @classmethod
    def build(cls, request, role, message=None):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
                "role": role,
                "message": message,
            },
        )

    context = [
        EntityResolve(key="request"),
        EntityResolve(key="request.created_by"),
        EntityResolve(key="request.receiver"),
    ]

    recipients = [
        UserRecipient(key="request.receiver"),
    ]

    recipient_filters = [
        UserPreferencesRecipientFilter(),
    ]

    recipient_backends = [
        UserEmailBackend(),
    ]
